## 快速上手

#### 示例

详细的内容在本页面的其他部分都会介绍，现在让我们先来从一个简单的例子入手，看看如何快速创建一个模块。

首先在`GOPATH`以外的地方创建一个目录，并初始化代码仓库（可选）：

```
$ mkdir -p /tmp/scratchpad/repo
$ cd /tmp/scratchpad/repo
$ git init -q
$ git remote add origin https://github.com/my/repo
```

初始化一个新的模块：

```
$ go mod init github.com/my/repo

go: creating new go.mod: module github.com/my/repo
```

写下你的代码：

```
$ cat <<EOF > hello.go
package main

import (
    "fmt"
    "rsc.io/quote"
)

func main() {
    fmt.Println(quote.Hello())
}
EOF
```

构建并运行：

```
$ go build -o hello
$ ./hello

Hello, world.
```

`go.mod`文件已经被自动更新了，包含了你的依赖包和版本号，版本号标签严格遵循[semver](https://semver.org/)规范。

```
$ cat go.mod

module github.com/my/repo

require rsc.io/quote v1.5.2
```

#### 日常使用流程

可以发现在上面的例子中，我们都没有使用到`go get`命令。

你的日常使用流程可以是：

- 在你的`.go`代码中根据需要添加`import`声明；
- 一些标准的命令例如`go build`或`go test`会自动添加新的依赖包以此来满足`imports`的需要（自动更新`go.mod`文件并且下载新的依赖包）；
- 如果需要的话，可以在`go get`命令或在编辑`go.mod`文件的时候指明版本号，例如`go get foo@v1.2.3`，`go get foo@master` (`foo@default`)，`go get foo@e3702bed2`。

其他你可能也经常会用到的功能也简单介绍一下：

- `go list -m all` —— 列出所有在build过程中会被直接和间接依赖的包的版本号；（[详情](https://github.com/golang/go/wiki/Modules#version-selection)）
- `go list -u -m all` —— 列出所有依赖包可用的升级；（[详情](https://github.com/golang/go/wiki/Modules#how-to-upgrade-and-downgrade-dependencies)）
- `go get -u ./...`或`go get -u=patch ./...`（在模块的根目录执行）—— 升级所有依赖包到最新版本（无视预发布的版本）；（[详情](https://github.com/golang/go/wiki/Modules#how-to-upgrade-and-downgrade-dependencies)）
- `go mod tidy` —— 从`go.mod`中清理所有不再需要的依赖包，并且增加其他需要的依赖包；（[详情](https://github.com/golang/go/wiki/Modules#how-to-prepare-for-a-release)）
- 在`go.mod`中使用`replace`可以使用其他依赖路径，比如fork的仓库、本地的拷贝或精确的版本号（例如`replace example.com/project/foo => ../foo`）；（[详情](https://github.com/golang/go/wiki/Modules#when-should-i-use-the-replace-directive)）
- `go mod vendor`一个可选的步骤，用于创建一个`vendor`路径。（[详情](https://github.com/golang/go/wiki/Modules#how-do-i-use-vendoring-with-modules-is-vendoring-going-away)）

在你读完接下来的“新概念”中的四小节内容后，足够应付大部分项目了。重新浏览目录也能帮助你熟悉Go Modules的一些更细节的讨论。
